"""Cog wrapper for command caching."""
import logging

from dis_snek import InteractionContext, Scale, Snake
from dis_snek.models.discord.embed import EmbedField
from jarvis_core.db import q
from jarvis_core.db.models import (
    Action,
    Ban,
    Kick,
    Modlog,
    Mute,
    Note,
    Setting,
    Warning,
)

from jarvis.utils import build_embed

MODLOG_LOOKUP = {"Ban": Ban, "Kick": Kick, "Mute": Mute, "Warning": Warning}
IGNORE_COMMANDS = {"Ban": ["bans"], "Kick": [], "Mute": ["unmute"], "Warning": ["warnings"]}


class ModcaseCog(Scale):
    """Cog wrapper for moderation case logging."""

    def __init__(self, bot: Snake):
        self.bot = bot
        self.logger = logging.getLogger(__name__)
        self.add_scale_postrun(self.log)

    async def log(self, ctx: InteractionContext, *_args: list, **kwargs: dict) -> None:
        """
        Log a moderation activity in a moderation case.

        Args:
            ctx: Command context
        """
        name = self.__name__.replace("Cog", "")

        if name in MODLOG_LOOKUP and ctx.command not in IGNORE_COMMANDS[name]:
            user = kwargs.pop("user", None)
            if not user and not ctx.target_id:
                self.logger.warning("Admin action %s missing user, exiting", name)
                return
            if ctx.target_id:
                user = ctx.target
            coll = MODLOG_LOOKUP.get(name, None)
            if not coll:
                self.logger.warning("Unsupported action %s, exiting", name)
                return

            action = await coll.find_one(q(user=user.id, guild=ctx.guild_id, active=True))
            if not action:
                self.logger.warning("Missing action %s, exiting", name)
                return

            action = Action(action_type=name.lower(), parent=action.id)
            note = Note(admin=self.bot.user.id, content="Moderation case opened automatically")
            await Modlog(user=user.id, admin=ctx.author.id, actions=[action], notes=[note]).commit()
            notify = await Setting.find_one(q(guild=ctx.guild.id, setting="notify", value=True))
            if notify and name not in ("Kick", "Ban"):  # Ignore Kick and Ban, as these are unique
                fields = (
                    EmbedField(name="Action Type", value=name, inline=False),
                    EmbedField(
                        name="Reason", value=kwargs.get("reason", None) or "N/A", inline=False
                    ),
                )
                embed = build_embed(
                    title="Admin action taken",
                    description=f"Admin action has been taken against you in {ctx.guild.name}",
                    fields=fields,
                )
                guild_url = f"https://discord.com/channels/{ctx.guild.id}"
                embed.set_author(name=ctx.guild.name, icon_url=ctx.guild.icon.url, url=guild_url)
                embed.set_thumbnail(url=ctx.guild.icon.url)
                await user.send(embed=embed)
